package no.ntnu.item.mopenid.webapp;
import java.io.IOException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import no.ntnu.item.mopenid.util.Utils;

import org.openid4java.message.AuthSuccess;
import org.openid4java.message.Message;
import org.openid4java.message.ParameterList;
import org.openid4java.message.VerifyResponse;
import org.openid4java.server.ServerException;
import org.openid4java.server.ServerManager;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.util.Log;




/* ------------------------------------------------------------ */
/** Hello Servlet 
 * 
 */
public class OpenIDServlet extends HttpServlet 
{
    Context androidContext;
    public ServerManager manager;
	private boolean authenticationFromProviderOkay;
	private boolean associateRequestReceived = false;
	private Message authenticationResponseMessage;
	private String identifier;
	private BroadcastReceiver authenticationReceiver;
	private HttpSession savedSession;
	private static final String INTENT_ACTION_SEND_AUTH_REQUEST = "no.ntnu.item.mopenid.ACTION_AUTH_REQUEST";
	private static final String INTENT_ACTION_SEND_AUTH_RESPONSE = "no.ntnu.item.mopenid.ACTION_AUTH_RESPONSE";
	private static final String INTENT_ACTION_SEND_AUTH_PRESSED_COMPELETE = "no.ntnu.item.mopenid.ACTION_AUTH_COMPLETED_LOGIN";
	private static final String INTENT_ACTION_SEND_AUTH_COMPLETED_COMPLETELY = "no.ntnu.item.mopenid.ACTION_AUTH_COMPLETED_LOGIN_COMPLETELY";
	private static final String INTENT_ACTION_SET_STATIC_IP_MODE = "no.ntnu.item.mopenid.ACTION_STATIC_IP_MODE";
	private static final String INTENT_ACTION_SET_STATIC_IP_MODE_REQUEST = "no.ntnu.item.mopenid.ACTION_STATIC_IP_MODE_REQUEST";
	private static final String IDENTIFIER_PREFIX = "https://mopenid.item.ntnu.no/id/";
	private static boolean staticIpMode = false;
	
    /* ------------------------------------------------------------ */
    public void init(ServletConfig config) throws ServletException
    {
    	super.init(config);
    	String endpointURL = Utils.getLocalEndpointURLAsString();
    	
    	manager = new ServerManager();
    	manager.setOPEndpointUrl(endpointURL);
    	
    	manager.setUserSetupUrl("/");
    	
    	Log.d("openid", "Set OP endpoint URL to: "+endpointURL);
    	authenticationFromProviderOkay = false;
        Object o = config.getServletContext().getAttribute("org.mortbay.ijetty.contentResolver");
        android.content.ContentResolver resolver = (android.content.ContentResolver)o;
        androidContext = (android.content.Context)config.getServletContext().getAttribute("org.mortbay.ijetty.context");
        
        setupStaticModeListener();
        modeResolver();
        
        Log.i("openid", "server address is: "+endpointURL);
    }

    private void modeResolver() {
		Intent intent = new Intent(INTENT_ACTION_SET_STATIC_IP_MODE_REQUEST);
		androidContext.sendBroadcast(intent);
	}

	private void setupStaticModeListener() {
		androidContext.registerReceiver(new BroadcastReceiver() {
			
			@Override
			public void onReceive(Context context, Intent intent) {
				if(intent.hasExtra("staticMode")) {
					staticIpMode = intent.getBooleanExtra("staticMode", false);
					Log.d("eirik", "Servlet received staticMode intent and it is: "+staticIpMode);
				}
				
			}
		}, new IntentFilter(INTENT_ACTION_SET_STATIC_IP_MODE));
		
	}

	/* ------------------------------------------------------------ */
    public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
        doGet(request, response);
    }

    /* ------------------------------------------------------------ */
    public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
    	
		
    	try {
    		
    		
    		
    		// Check to see if the IP address of the smartphone have changed since last time
    		// Happens only in the rare case of the smartphone IP going from public to hidden, then back to 
    		// public again, without restarting the iJetty webserver.
    		String endpointURL = Utils.getLocalEndpointURLAsString(); // quite time consuming for each http request.
    		if(!endpointURL.equals(manager.getOPEndpointUrl())) {
    			Log.d("eirik", "OP Endpoint URL have changed => updating...");
    			manager.setOPEndpointUrl(endpointURL);
    		}
    		
    		
    		processRequestAndReturnResponse(request, response);
    	   		
    	} catch (Exception e) {
    		Log.i("openid", "ukoselig feilmelding.");
    		e.printStackTrace();
    		
    		response.setContentType("text/html");
    		ServletOutputStream out = response.getOutputStream();
    		out.println("error dessverre: "+ e.toString()); //TODO better error handling
    		out.flush();
		}
        
        
    }

	private void processRequestAndReturnResponse(HttpServletRequest request, HttpServletResponse response) throws Exception{
		
		// extract the parameters from the request
        ParameterList requestParameters = new ParameterList(request.getParameterMap());
        

        String mode = requestParameters.hasParameter("openid.mode") ?
                requestParameters.getParameterValue("openid.mode") : null;
                
        Log.d("openid", "Mode is: "+mode);

        
        Message responseMessage;
        String responseText;
        
        if(mode == null) {
        	// request comes from "complete login" form, or is a RP performing OP endpoint localization, or is an xrds request

			if (savedSession == null) {
				if (staticIpMode) {
					if (request.getRequestURI().contains("/xrds")) {
						buildAndPrintXrdsDocument(response);
					} else {
						buildAndPrintIdPage(response);
					}
				}
				return;

			}
        		
        	
        	Log.d("openid", "is the session the same ? "+request.getSession().getId().equals(savedSession.getId()));
        	

        	if(savedSession != null) {
        		requestParameters = (ParameterList) savedSession.getAttribute("requestParameters");
        		savedSession.removeAttribute("requestParameters");
        	}
        	
//        	Log.d("openid", "requestParameters from session: "+requestParameters.toString());
        	
        	boolean isAuthenticatedAndHaveSameSession = authenticationFromProviderOkay && request.getSession().getId().equals(savedSession.getId());
        	
        	authenticationResponseMessage = manager.authResponse(requestParameters,
            		identifier, identifier, isAuthenticatedAndHaveSameSession);
        	
        	
        	if(isAuthenticatedAndHaveSameSession) {
        		//login used, notify mopenid application with intent
        		Intent confirmationIntent = new Intent(INTENT_ACTION_SEND_AUTH_PRESSED_COMPELETE);
        		androidContext.sendBroadcast(confirmationIntent);
        	}
        	

        	
        	String redirectURL = authenticationResponseMessage.getDestinationUrl(true);

        	Log.d("eirik", "destination url: " + redirectURL);
        	response.sendRedirect(redirectURL);
        	savedSession = null;
        	identifier = null;
        	
        	if(authenticationResponseMessage instanceof AuthSuccess) {
        		if(associateRequestReceived) {
        			new DelayThread(6000).start();
        		}
        	}

        	authenticationResponseMessage = null;
        	associateRequestReceived = false;
        }
        
        
        else if ("associate".equals(mode))
        {
        	
        	
            // --- handle an association request ---
        	associateRequestReceived = true;
            handleAssociationRequest(response, requestParameters);
            
        }
        else if ("checkid_setup".equals(mode))
        {
            requestUserAuthenticationAndRegisterReceiver(requestParameters);

            // --- process an authentication request ---
//            AuthRequest authReq = AuthRequest.createAuthRequest(requestParameters, manager.getRealmVerifier());
            
            assignIdentifierAndSessionField(request, requestParameters);
            printFinalLoginForm(identifier, response);
            
        }
        else if("checkid_immediate".equals(mode)) {
        	associateRequestReceived = false;
        	assignIdentifierAndSessionField(request, requestParameters);
        	Message authImmediateFailureMessage = manager.authResponse(requestParameters, identifier, identifier, false);
        	String openidNs = authImmediateFailureMessage.getParameterValue("openid.ns");
        	String openidMode = authImmediateFailureMessage.getParameterValue("openid.mode");
        	
        	String redirectURL = authImmediateFailureMessage.getDestinationUrl(false) +
        			"&openid.mode="+openidMode+"&openid.ns="+openidNs;
        	
        	Log.d("eirik", "destination url: " + redirectURL);
    		response.sendRedirect(redirectURL);
          
    		Log.i("openid", "Sent authImmediateFailureMessage message to RP");
        	
        }
        else if ("check_authentication".equals(mode))
        {
            // --- processing a verification request ---
            responseMessage = manager.verify(requestParameters);
            responseText = responseMessage.keyValueFormEncoding();
            
            ServletOutputStream os = response.getOutputStream();
    		os.write(responseText.getBytes());
    		os.flush();
    		Log.i("openid", "Sent verification message to RP");
    		if(responseMessage instanceof VerifyResponse) {
    			VerifyResponse msg = (VerifyResponse) responseMessage;
        		if(!associateRequestReceived && msg.isSignatureVerified()) {
        			
        			DelayThread delayedExecution = new DelayThread(5000);					
					delayedExecution.start();
					
    			}
    		}
    		
        }
        else
        {
            // --- error response ---
//            responseMessage = DirectError.createDirectError("Unknown request");
//            responseText = responseMessage.keyValueFormEncoding();
        	Log.d("openid", "unknown openid request");
        	
        }
        
		
	}
	
	private class DelayThread extends Thread {
		private int delay;
		public DelayThread(int delay) {
			super();
			this.delay = delay;
			
		}
		
		public void run() {
			try {
				sleep(delay);
			} catch (InterruptedException e) {
				e.printStackTrace();
				Log.e("eirik","Interrupted: "+e.getMessage());
			}
			Intent allGoodIntent = new Intent(INTENT_ACTION_SEND_AUTH_COMPLETED_COMPLETELY);
			androidContext.sendBroadcast(allGoodIntent);
			Log.d("eirik", "Sent Intent to close confirmation dialog, "+(int)delay/1000+" seconds after.");
			
			
		}
	}
	

	private void buildAndPrintIdPage(HttpServletResponse response) throws IOException {
		
		ServletOutputStream os = response.getOutputStream();
		String htmlDocument = Utils.readFileAsString("/mnt/sdcard/jetty/webapps/mopenid/id_page.html");
		String opEndpointURL = manager.getOPEndpointUrl();
		htmlDocument = htmlDocument.replaceAll("%s", opEndpointURL);
		htmlDocument = htmlDocument.replaceAll("%x", opEndpointURL+"/xrds");
		os.println(htmlDocument);
		os.flush();
		
	}
	
	private void buildAndPrintXrdsDocument(HttpServletResponse response) throws IOException {
		
		response.setContentType("application/xrds+xml");
		ServletOutputStream os = response.getOutputStream();
		String htmlDocument = Utils.readFileAsString("/mnt/sdcard/jetty/webapps/mopenid/xrds");
		String opEndpointURL = manager.getOPEndpointUrl();
		htmlDocument = htmlDocument.replaceAll("%s", opEndpointURL);
		os.println(htmlDocument);
		os.flush();
		
	}


	private void assignIdentifierAndSessionField(HttpServletRequest request,
			ParameterList requestParameters) {
		identifier = requestParameters.getParameterValue("openid.identity");
		String claimedIdentifier = requestParameters.getParameterValue("openid.claimed_identifier");
		savedSession = request.getSession();
		savedSession.setAttribute("identifier", identifier);
		savedSession.setAttribute("claimedIdentifier", claimedIdentifier);
		savedSession.setAttribute("return_to", requestParameters.getParameterValue("openid.return_to"));
		savedSession.setAttribute("requestParameters", requestParameters);
	}

	

	private void printErrorMessage(HttpServletResponse response) throws IOException {
		ServletOutputStream os = response.getOutputStream();
		os.println("<html>" +
				"<head>" +
				"	<title>mOpenID</title>" +
				"</head>" +
				"<body>Stupid error message" +
				"</body></html>");
		os.flush();
		
	}

	private void printFinalLoginForm(String identifier,	HttpServletResponse response) throws IOException {
		ServletOutputStream os = response.getOutputStream();
		String htmlDocument = Utils.readFileAsString("/mnt/sdcard/jetty/webapps/mopenid/login_screen.html");
		htmlDocument = htmlDocument.replaceFirst("%s", identifier);
		os.println(htmlDocument);
		os.flush();
		
	}

	private void handleAssociationRequest(HttpServletResponse response, ParameterList requestParameters) throws IOException {
		Message responseMessage;
		String responseText;
		responseMessage = manager.associationResponse(requestParameters);
		responseText = responseMessage.keyValueFormEncoding();
		
		//write output to the RP
		
		ServletOutputStream os = response.getOutputStream();
		os.write(responseText.getBytes());
		os.flush();
		Log.i("openid", "RP sent an associate request, and this is the reply: "+responseText);
	}
	
	 private void requestUserAuthenticationAndRegisterReceiver(ParameterList request) throws ServerException
	    {

	    	setupAndRegisterAuthenticationReceiver();
	    	
	    	Intent infoIntent = new Intent(INTENT_ACTION_SEND_AUTH_REQUEST);
	    	infoIntent.putExtra("openid.realm", request.getParameterValue("openid.realm"));
	    	infoIntent.putExtra("openid.claimed_id", request.getParameterValue("openid.claimed_id"));
	    	
	    	androidContext.sendBroadcast(infoIntent); //TODO: add permission required
	    	
	    	Log.i("openid", "Sent Intent ala "+INTENT_ACTION_SEND_AUTH_REQUEST);
	    	
	    	
	    	
	    	
	    }

	private void setupAndRegisterAuthenticationReceiver() {
		
		authenticationReceiver = new BroadcastReceiver() {
			
			

			@Override
			public void onReceive(Context context, Intent intent) {
				authenticationFromProviderOkay = intent.getBooleanExtra("authenticated", false);
				identifier = intent.getStringExtra("identifier");
				Log.i("openid", "recived auth response and it was: "+authenticationFromProviderOkay);
				if(!authenticationFromProviderOkay) {
					String reason = intent.getStringExtra("reason");
					Log.i("openid", "The reason is: "+reason);
				}
				androidContext.unregisterReceiver(authenticationReceiver);
			}
		};
		
		androidContext.registerReceiver(authenticationReceiver, new IntentFilter(INTENT_ACTION_SEND_AUTH_RESPONSE));
		
		
	}
    
}
