package no.ntnu.item.mopenid.app;

import java.util.Calendar;

import no.ntnu.item.mopenid.util.OpenID;
import no.ntnu.item.mopenid.util.Utils;
import android.app.Activity;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Color;
import android.os.AsyncTask;
import android.os.Bundle;
import android.text.format.DateFormat;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

public abstract class AbstractActivation extends Activity {

	protected static final int CANCEL_LOGIN_DIALOG = 0;
	protected static final int SETUP_PROGRESS_DIALOG = 1;
	protected static final int AUTHENTICATION_DIALOG = 2;
	protected static final int AUTHENTICATION_CONFIRMATION_DIALOG = 3;
	protected static final int CLAIMED_ID_MISMATCH = 4;
	protected static final String INTENT_ACTION_AUTH_REQUEST = "no.ntnu.item.mopenid.ACTION_AUTH_REQUEST";
	protected static final String INTENT_ACTION_AUTH_RESPONSE = "no.ntnu.item.mopenid.ACTION_AUTH_RESPONSE";
	protected static final String INTENT_ACTION_AUTH_COMPLETED = "no.ntnu.item.mopenid.ACTION_AUTH_COMPLETED_LOGIN";
	protected static final String INTENT_ACTION_AUTH_COMPLETELY_COMPLETED = "no.ntnu.item.mopenid.ACTION_AUTH_COMPLETED_LOGIN_COMPLETELY";
	
	protected ProgressDialog progressDialog;
	protected BroadcastReceiver receiver;
	boolean linkedWithmOpenID = false;
	protected String identifier = null;
	protected Intent receivedIntent;
	protected int currentShowingDialogId;

	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activation_screen);
		receiver = null;
		setUpCancelButtonBehavior();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		menu.add(Menu.NONE, 1, Menu.NONE, "Simulate authentication request");
		return super.onCreateOptionsMenu(menu);
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {

		switch (item.getItemId()) {
		case 1:
			showDialog(AUTHENTICATION_DIALOG);
			return false;

		default:
			return super.onOptionsItemSelected(item);
		}
	}

	protected abstract void setUpCancelButtonBehavior();

	@Override
	protected void onPause() {
		unregisterReceiver(receiver);
		super.onPause();
	}

	@Override
	protected void onResume() {

		receiver = new BroadcastReceiver() {

			@Override
			public void onReceive(Context context, Intent intent) {
				receivedIntent = intent;
				if((intent.getStringExtra("openid.claimed_id").equalsIgnoreCase(identifier) && AbstractActivation.this instanceof StaticIpModeActivation)
						|| (intent.getStringExtra("openid.claimed_id").endsWith(identifier) && AbstractActivation.this instanceof MOpenIdModeActivation)) {
					showDialog(AUTHENTICATION_DIALOG);
				} else {
					sendAuthReponse(false, "claimed id mismatch");
					Log.d("eirik", "openid.identifier: "+intent.getStringExtra("openid.claimed_id")+ " and identifier: "+identifier);
					showDialog(CLAIMED_ID_MISMATCH);
				}
				

			}
		};

		registerReceiver(receiver, new IntentFilter(INTENT_ACTION_AUTH_REQUEST));
		super.onResume();
	}

	protected abstract void showOpenIDURLAndCancelButton();


	protected void setAllTextViewsToSuccessful() {
		ProgressBar progressBar = (ProgressBar) findViewById(R.id.progressbar_activating_id);
		progressBar.setVisibility(View.GONE);
		TextView activationHeading = (TextView) findViewById(R.id.activation_heading);
		activationHeading.setText(R.string.id_activated);
		activationHeading.setTextColor(Color.rgb(0, 153, 0));

		LinearLayout layout = (LinearLayout) findViewById(R.id.layout_activation_heading);
		layout.requestLayout();
	}

	protected void startLocalOpenIDProvider() {
		// TODO: Make working so that iJetty service works well
		Intent intent = new Intent("hello");
		intent.putExtra(Utils.__PORT, Utils.__PORT_DEFAULT);
		intent.putExtra(Utils.__NIO, Utils.__NIO_DEFAULT);
		intent.putExtra(Utils.__SSL, Utils.__SSL_DEFAULT);
		intent.putExtra(Utils.__CONSOLE_PWD, Utils.__CONSOLE_PWD_DEFAULT);
		startService(intent);
		Log.i("eirik", "Local OP started");
	}
	

	@Override
	protected Dialog onCreateDialog(int id) {
		Dialog dialog;
		currentShowingDialogId = id;
		switch (id) {
		case CANCEL_LOGIN_DIALOG:
			dialog = new Dialog(this);
			
			dialog.setContentView(R.layout.popup_disassociation);
			dialog.setTitle(R.string.closing_application);
			dialog.setCanceledOnTouchOutside(false);
			break;

		case SETUP_PROGRESS_DIALOG:
			progressDialog = new ProgressDialog(this);
			dialog = progressDialog;
			dialog.setCancelable(false);
			dialog.setCanceledOnTouchOutside(false);
			setupProgressDialog();
			break;

		case AUTHENTICATION_DIALOG:
			dialog = new Dialog(this);
			setupAuthenticationDialog(dialog);
			break;

		case AUTHENTICATION_CONFIRMATION_DIALOG:
			dialog = new Dialog(this);
			setupAuthenticationConfirmationDialog(dialog);

			break;

		case CLAIMED_ID_MISMATCH:
			dialog = new Dialog(this);
			setupClaimedIdMismatchDialog(dialog);
			break;
		default:
			dialog = null;
			currentShowingDialogId = -1;
		}
		return dialog;
	}


	protected abstract void setupProgressDialog();

	private void setupClaimedIdMismatchDialog(Dialog dialog) {
		dialog.setContentView(R.layout.claimed_id_mismatch_dialog);
		dialog.setTitle(R.string.invalid_request_heading);
		dialog.setCancelable(false);
		dialog.setCanceledOnTouchOutside(false);
		Button okayButton = (Button) dialog
		.findViewById(R.id.mismatch_okay_button);
		okayButton.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				perfromCancelAction();
				
			}
		});

	}

	protected abstract void perfromCancelAction();

	private void setupAuthenticationConfirmationDialog(Dialog dialog) {
		dialog.setContentView(R.layout.authentication_confirmation_dialog);
		dialog.setTitle(R.string.auth_confirmation_heading);
		dialog.setCanceledOnTouchOutside(false);
		dialog.setCancelable(false);
		
		
		Button okayButton = (Button) dialog
				.findViewById(R.id.authentication_confirm_okay_button);
		okayButton.setVisibility(View.GONE); // TODO temp
		okayButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				
			}
		});

	}
	
	protected abstract void authenticationFinished();
	
	private void sendAuthReponse(boolean approved, String reason) {
		Intent intent = new Intent(INTENT_ACTION_AUTH_RESPONSE);
		intent.putExtra("authenticated", approved);
		intent.putExtra("identifier", (this instanceof MOpenIdModeActivation) ? Utils.MOPENID_URL + identifier : Utils.getLocalEndpointURLAsString());
		
		if(!approved) {
			intent.putExtra("reason", reason);
		} else {
			
			setupAuthenticationCompletelyCompletedReceiver();
			
			
			unregisterReceiver(receiver);
			receiver = new BroadcastReceiver() {
				
				@Override
				public void onReceive(Context context, Intent intent) {
					// update last used
					if(AbstractActivation.this instanceof MOpenIdModeActivation) {
						OpenID openId = new OpenID(identifier);
						String lastUsed = (String) DateFormat.format("yyyy-MM-dd kk:mm:ss", Calendar.getInstance());
						Log.d("eirik", "set last used to: "+lastUsed);
						openId.setLastUsed(lastUsed);
						Utils.dbUpdateLastUsed(openId);
					}
				}
			};
			
			registerReceiver(receiver, new IntentFilter(INTENT_ACTION_AUTH_COMPLETED));
		}
		sendBroadcast(intent);
		Log.i("openid", "Sent Intent ala "+INTENT_ACTION_AUTH_RESPONSE);
	}

	private void setupAuthenticationCompletelyCompletedReceiver() {
		BroadcastReceiver completelyReceiver = new BroadcastReceiver() {
			
			@Override
			public void onReceive(Context context, Intent intent) {
				authenticationFinished();
			}
		};
		
		registerReceiver(completelyReceiver, new IntentFilter(INTENT_ACTION_AUTH_COMPLETELY_COMPLETED));
	}

	private void setupAuthenticationDialog(Dialog dialog) {
		dialog.setContentView(R.layout.authentication_dialog);
		dialog.setCanceledOnTouchOutside(false);
		dialog.setCancelable(false);

		TextView realmTextView = (TextView) dialog
				.findViewById(R.id.auth_realm);
		realmTextView.setText(receivedIntent.getStringExtra("openid.realm"));
		TextView identifierTextView = (TextView) dialog
				.findViewById(R.id.auth_identifier);
		identifierTextView.setText(identifier);

		Button yesButton = (Button) dialog
				.findViewById(R.id.authenticate_yes_button);
		yesButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				Log.i("eirik", "clicked yes for authentication");
				removeDialog(AUTHENTICATION_DIALOG);
				sendAuthReponse(true, null);
				showDialog(AUTHENTICATION_CONFIRMATION_DIALOG);
			}
			
		});

		Button noButton = (Button) dialog
				.findViewById(R.id.authenticate_no_button);
		noButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				perfromCancelAction();
				sendAuthReponse(false, "cancelled");
				Log.i("eirik", "Clicked no button");
			}
		});

	}
	

}