package no.ntnu.item.mopenid.app;


import no.ntnu.item.mopenid.util.Utils;

import org.json.JSONException;
import org.json.JSONObject;

import android.app.Activity;
import android.content.Intent;
import android.graphics.Color;
import android.os.AsyncTask;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;



public class CreateNew extends Activity {

	private final static int ID_MINIMUM_LENGTH = 5;
	
	EditText editNewField ;
	LinearLayout newidLayout;

	@Override 
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		setContentView(R.layout.create_new);

		newidLayout = (LinearLayout) findViewById(R.id.layout_newid);
		editNewField = (EditText) findViewById(R.id.create_new_field);

		TextView openIdURL = (TextView) findViewById(R.id.openid_identifier_url);
		openIdURL.setTextColor(Color.RED);
		final Button checkAvailabilityButton = setAvailabilityButtonBehavior();
		checkAvailabilityButton.setClickable(false);
		EditText editNewField = setEditTextBehavior(openIdURL,
				checkAvailabilityButton);

	}

	private Button setAvailabilityButtonBehavior() {

		final Button checkAvailabilityButton = (Button) findViewById(R.id.check_availability_button);
		
		checkAvailabilityButton.setOnClickListener(new OnClickListener() {

//			@Override
			public void onClick(View v) {
				final String desiredOpenID = editNewField.getText().toString();
				if(desiredOpenID.length() >= ID_MINIMUM_LENGTH) {
					CreateNew.this.setButtonOnClickBehavior((Button) v);
				} else {
					Toast.makeText(getApplicationContext(), "Minimum "+ID_MINIMUM_LENGTH+" characters",
							Toast.LENGTH_SHORT).show();
				}
				
			}
		});

		return checkAvailabilityButton;

	}
	
	
	private void setButtonOnClickBehavior(Button checkAvailabilityButton) {
		checkAvailabilityButton.setVisibility(View.GONE);
		final ProgressBar progressBar = (ProgressBar) findViewById(R.id.progressBar_checkid);
		progressBar.setVisibility(View.VISIBLE);

//		final EditText editNewField = (EditText) findViewById(R.id.create_new_field);
		final String desiredOpenID = editNewField.getText().toString();

		new CheckAvailabilityTask().execute(desiredOpenID);
		Log.i("eirik", "trying to fetch result");
		
		
		
	}

	private EditText setEditTextBehavior(final TextView openIdURL,
			final Button checkAvailabilityButton) {

		final EditText editNewField = (EditText) findViewById(R.id.create_new_field);

		// set the keyboard to automatically show
		editNewField.setOnFocusChangeListener(new View.OnFocusChangeListener() {
			public void onFocusChange(View v, boolean hasFocus) {
				if (hasFocus) {
					getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_VISIBLE);
				}
			}
		});
		
		

		editNewField.addTextChangedListener(new TextWatcher() {

			public void onTextChanged(CharSequence s, int start, int before,
					int count) {
				CharSequence urlTextCurrent = openIdURL.getText();
				checkAvailabilityButton.setText(R.string.check_availability);
				setAvailabilityButtonBehavior();

				if (s.length() == 0) {
					checkAvailabilityButton.setClickable(false);
					openIdURL.setTextColor(Color.rgb(255, 0, 0)); // red
					checkAvailabilityButton.setTextColor(Color.rgb(119, 119,
							119));// grey
				} else if (ID_MINIMUM_LENGTH > s.length()) {
					openIdURL.setTextColor(Color.rgb(255, 215, 0)); // orange
					checkAvailabilityButton.setClickable(true);
					checkAvailabilityButton.setTextColor(Color.rgb(119, 119,
							119)); // grey
				} else {
					openIdURL.setTextColor(Color.rgb(255, 255, 0)); // yellow
					checkAvailabilityButton.setClickable(true);
					checkAvailabilityButton.setTextColor(Color.rgb(0, 153, 0)); // green
				}

				if (count < before) {
					// must remove one character
					openIdURL.setText(urlTextCurrent.toString().substring(0,
							openIdURL.length() - 1));
				} else {
					// must add
					openIdURL.setText(urlTextCurrent.toString()
							+ s.toString().substring(start));
				}

			}

			public void beforeTextChanged(CharSequence s, int start, int count,
					int after) {

			}

			public void afterTextChanged(Editable s) {

			}
		});

		return editNewField;
	}

	private class CheckAvailabilityTask extends AsyncTask<String, Void, String> {
		private static final String STATUS_OK = "no";
		private static final String STATUS_INVALID = "invalid_id";
		private static final String STATUS_TAKEN = "yes";
		private String identifier = null;

		protected String doInBackground(String... identifiers) {
			identifier = identifiers[0];
			
			
			return isOpenIDAvailable();
		}

		private String isOpenIDAvailable() {
			return Utils.isAvailable(identifier);
		}

		protected void onPostExecute(String result) {
			final ProgressBar progressBar = (ProgressBar) findViewById(R.id.progressBar_checkid);
			progressBar.setVisibility(View.GONE);
			final Button checkAvailabilityButton = (Button) findViewById(R.id.check_availability_button);
			checkAvailabilityButton.setVisibility(View.VISIBLE);
			
			try {
				JSONObject response = new JSONObject(result);
				
				if(response.get("message").equals(STATUS_OK)) {
					
					checkAvailabilityButton.setText(R.string.button_available);
					TextView openIdURL = (TextView) findViewById(R.id.openid_identifier_url);
					openIdURL.setTextColor(Color.rgb(0, 153, 0));
					checkAvailabilityButton.setTextColor(Color.rgb(0, 153, 0));
					checkAvailabilityButton.setOnClickListener(new OnClickListener() {
						
						public void onClick(View v) {
							editNewField.setEnabled(false);
							editNewField.setFocusable(false);
							checkAvailabilityButton.setVisibility(View.GONE);
							progressBar.setVisibility(View.VISIBLE);
							
							boolean registerSuccess = false;
							registerSuccess = Utils.registerNewID(identifier);
							
							if(registerSuccess) {
								Log.i("eirik", "insertions success");
								Intent selectmOpenIDActivity = new Intent(v.getContext(), SelectMOpenIDMode.class);
								selectmOpenIDActivity.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
								startActivity(selectmOpenIDActivity);
							} else {
								Log.i("eirik", "insertion failure");
								//TODO: handle error
							}
							
						}
						
					});
					
					
				} else if (response.get("message").equals(STATUS_INVALID)) {
					Log.i("eirik", "Identifier is invalid. Only certain characters are allowed.");
					Toast.makeText(getApplicationContext(), "Invalid characters used.",
							Toast.LENGTH_SHORT).show();
				} else if (response.get("message").equals(STATUS_TAKEN)) {
					TextView openIdURL = (TextView) findViewById(R.id.openid_identifier_url);
					openIdURL.setTextColor(Color.RED);
					checkAvailabilityButton.setClickable(false);
					checkAvailabilityButton.setTextColor(Color.RED);
					checkAvailabilityButton.setText(R.string.button_available_taken);
					
				} else {
					throw new JSONException("some weird (assumed) json response received: "+ result);
				}
				
			} catch (JSONException e) {
				e.printStackTrace();
				Toast.makeText(getApplicationContext(), result,
						Toast.LENGTH_SHORT).show();
			}
			
			
			
		}
		
		
	}
}