package no.ntnu.item.mopenid.app;

import static no.ntnu.item.mopenid.app.AbstractActivation.CANCEL_LOGIN_DIALOG;
import no.ntnu.item.mopenid.util.Utils;
import android.app.ProgressDialog;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;

public class MOpenIdModeActivation extends AbstractActivation {

	
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		Bundle extras = getIntent().getExtras();
		identifier = (String) extras.getString("id");
	}

	@Override
	protected void setUpCancelButtonBehavior() {
		Button cancelButton = (Button) findViewById(R.id.cancelButton);
		cancelButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				Log.i("eirik", "Cancel button clicked");

				new CancelButtonWithDisassociationTask().execute(identifier);

			}

		});
		
	}

	private class CancelButtonWithDisassociationTask extends
			AsyncTask<String, Void, Boolean> {
		private String identifier = null;

		@Override
		protected void onPreExecute() {
			removeDialog(currentShowingDialogId);
			showDialog(CANCEL_LOGIN_DIALOG);
		}

		protected Boolean doInBackground(String... identifiers) {
			identifier = identifiers[0];
			return Utils.disassociateOpenID(identifier);
		}

		protected void onPostExecute(Boolean disassociationSuccessful) {

			if (disassociationSuccessful.booleanValue()) {
				Log.i("eirik", "successful disassociation");
				removeDialog(CANCEL_LOGIN_DIALOG);
				setResult(SelectMode.RESULT_AUTHENTICATION_CANCELLATION);
				finish();
			} else {
				Log.i("eirik", "unsuccessful disassociation");
				removeDialog(CANCEL_LOGIN_DIALOG);
			}

		}

	}

	@Override
	protected void onPause() {
		super.onPause();
		Utils.disassociateOpenID(identifier);
	}
	
	
	@Override 
	protected void onResume() {
		showDialog(SETUP_PROGRESS_DIALOG);
		new ActivateIdentityProviderAndLinkWithmOpenIdTask().execute();
		super.onResume();
	}
	
	private class AuthenticationFinishedWithDisassociationTask extends
			CancelButtonWithDisassociationTask {

		@Override
		protected void onPreExecute() {
			removeDialog(AUTHENTICATION_CONFIRMATION_DIALOG);
		}

		protected void onPostExecute(Boolean disassociationSuccessful) {
			if (disassociationSuccessful.booleanValue()) {
				Log.i("eirik", "successful disassociation");
				setResult(SelectMode.RESULT_AUTHENTICATION_SUCCESSFUL);
				removeDialog(CANCEL_LOGIN_DIALOG);
				finish();
			} else {
				Log.i("eirik", "unsuccessful disassociation");
			}

		}
	}

	@Override
	protected void authenticationFinished() {
		new AuthenticationFinishedWithDisassociationTask().execute(identifier);

	}

	private class ActivateIdentityProviderAndLinkWithmOpenIdTask extends
			AsyncTask<Void, Void, Void> {

		@Override
		protected void onPreExecute() {
			progressDialog.setProgress(0);
		}

		@Override
		protected Void doInBackground(Void... params) {

			startLocalOpenIDProvider();
			progressDialog.setProgress(40);
			connectToAndRedirectmOpenIDServerToHere();
			progressDialog.setProgress(100);
			return null;
		}

		@Override
		protected void onPostExecute(Void result) {
			setAllTextViewsToSuccessful();
			showOpenIDURLAndCancelButton();
			removeDialog(AbstractActivation.SETUP_PROGRESS_DIALOG);
			linkedWithmOpenID = true;
		}

	}
	

	protected void connectToAndRedirectmOpenIDServerToHere() {
		linkedWithmOpenID = Utils.linkGivenmOpenIDtoHere(identifier);

	}
	
	protected void setupProgressDialog() {
		progressDialog.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
		progressDialog
				.setMessage("Activating Identity Provider and linking with mOpenID server, please wait...");
		progressDialog.setCancelable(false);
		progressDialog.setCanceledOnTouchOutside(false);
	}

	@Override
	protected void perfromCancelAction() {
		new CancelButtonWithDisassociationTask().execute(identifier);
		
	}

	@Override
	protected void showOpenIDURLAndCancelButton() {
		TextView usageInformationTextView = (TextView) findViewById(R.id.usage_instructions), openidURLTextView = (TextView) findViewById(R.id.openid_to_use);
		usageInformationTextView.setVisibility(View.VISIBLE);
		openidURLTextView.setText(Utils.MOPENID_URL + identifier);
		openidURLTextView.setVisibility(View.VISIBLE);

		Button cancelButton = (Button) findViewById(R.id.cancelButton);
		cancelButton.setVisibility(View.VISIBLE);

	}
}
