package no.ntnu.item.mopenid.app;

import java.util.Calendar;
import java.util.List;

import no.ntnu.item.mopenid.util.OpenID;
import no.ntnu.item.mopenid.util.Utils;
import android.app.AlertDialog;
import android.app.ListActivity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.media.AudioRecord.OnRecordPositionUpdateListener;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.text.format.DateFormat;
import android.util.Log;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView.OnItemLongClickListener;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

public class SelectMOpenIDMode extends ListActivity {

	public static final String __PORT = "org.mortbay.ijetty.port";
	public static final String __NIO = "org.mortbay.ijetty.nio";
	public static final String __SSL = "org.mortbay.ijetty.ssl";

	public static final String __CONSOLE_PWD = "org.mortbay.ijetty.console";
	public static final String __PORT_DEFAULT = "8080";
	public static final boolean __NIO_DEFAULT = true;
	public static final boolean __SSL_DEFAULT = false;

	public static final String __CONSOLE_PWD_DEFAULT = "admin";

	private ArrayAdapter<OpenID> adapter;
	private OpenID idToDelete = null;
	
	private static final int MENU_OPTION_DELETE_COMPLETELY = 1;
	private static final int MENU_OPTION_DELETE_LOCAL_ONLY = 2;

	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		Utils.setApplicationContext(getApplicationContext());

		setContentView(R.layout.select_mopenid);
		setUpListBehavior();

	}
	
	

	private void setUpListBehavior() {
		List<OpenID> linkIDs = Utils.getSavedOpenIDs();

		if (linkIDs.size() <= Utils.MAX_NUMBER_OF_IDS)
			linkIDs.add(new OpenID(">> Create new"));
		adapter = new MyArrayAdapter<OpenID>(this, R.layout.id_list_item,
				linkIDs);
		setListAdapter(adapter);

		ListView lv = getListView();
		

		lv.setTextFilterEnabled(true);

		lv.setOnItemClickListener(new OnItemClickListener() {

			public void onItemClick(AdapterView<?> parent, View view,
					int position, long id) {
				OpenID openid = (OpenID) parent.getItemAtPosition(position);

				if (!openid.isCreateNewButton()) {

					Intent intent = new Intent(SelectMOpenIDMode.this,
							MOpenIdModeActivation.class);
					intent.putExtra("id", openid.getIdentifier());
					intent.addFlags(Intent.FLAG_ACTIVITY_FORWARD_RESULT); //TODO: mention in report. nice move!
					intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
					startActivity(intent);

				} else {
					// open new activity if create new was pushed
					Intent startActivityIntent = new Intent(view.getContext(),
							CreateNew.class);
					startActivity(startActivityIntent);
				}

			}

		});
		
		registerForContextMenu(lv);
	}
	
	@Override
	public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
		
		AdapterView.AdapterContextMenuInfo info = (AdapterView.AdapterContextMenuInfo)menuInfo;
		List<OpenID> list = Utils.getSavedOpenIDs();
		idToDelete = list.get(info.position);
	    menu.setHeaderTitle(idToDelete.getIdentifier());
	    
		menu.add(Menu.NONE, MENU_OPTION_DELETE_COMPLETELY , Menu.NONE, "Delete completely");
		menu.add(Menu.NONE, MENU_OPTION_DELETE_LOCAL_ONLY , Menu.NONE, "Delete locally only");
	}
	
	@Override
	public boolean onContextItemSelected(MenuItem item) {
		
		switch (item.getItemId()) {
		case MENU_OPTION_DELETE_COMPLETELY:
			showDeleteAlertDialog(item.getItemId(), true);
			break;

		case MENU_OPTION_DELETE_LOCAL_ONLY:
			showDeleteAlertDialog(item.getItemId(), false);
			
			break;
		default:
			break;
		}
		
		
	  return true;
	}

	private void showDeleteAlertDialog(final int itemId, final boolean deleteRemotelyToo) {
		
		new AlertDialog.Builder(SelectMOpenIDMode.this)
		.setIcon(R.drawable.ic_delete)
		.setTitle(R.string.are_you_sure)
		.setMessage(R.string.delete_info_text)
		.setPositiveButton(R.string.button_yes, new DialogInterface.OnClickListener() {
			
			@Override
			public void onClick(DialogInterface dialog, int which) {
				setUpListBehavior();
				 if(Utils.deleteId(idToDelete, deleteRemotelyToo)) {
			        	setUpListBehavior();
			        	Toast.makeText(getApplicationContext(), (deleteRemotelyToo) ? R.string.deleted : R.string.deleted_locally ,Toast.LENGTH_SHORT).show();
			        } else {
			        	Toast.makeText(getApplicationContext(), R.string.id_not_deleted ,Toast.LENGTH_SHORT).show();
			        }

					idToDelete = null;
			}

		})
		.setNegativeButton(R.string.button_no, null)
		.show();
		

	}
	

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		setResult(resultCode);
		finish();
		
	}

	private class MyArrayAdapter<T> extends ArrayAdapter<T> {

		public MyArrayAdapter(Context context, int textViewResourceId,
				List<T> objects) {
			super(context, textViewResourceId, objects);
			
		}

		public View getView(int position, View listViewItem, ViewGroup parent) {
			View v = listViewItem;
			if (v == null) {
				LayoutInflater vi = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
				v = vi.inflate(R.layout.openid_list_item, null);
			}

			TextView twId = (TextView) v.findViewById(R.id.openid_identifier);

			TextView twLastUsed = (TextView) v.findViewById(R.id.openid_lastused);
			OpenID openID = (OpenID) getItem(position);

			twId.setText(openID.getIdentifier());

			if (!openID.getIdentifier().equals(">> Create new")) {
				try {
					Calendar cal = Utils.parseTimestamp(openID.getLastUsed());
					twLastUsed.setText("(last used "
							+ DateFormat.format("dd.MM.yyyy kk:mm:ss", cal)
							+ ")");
				} catch (Exception e) {
					twLastUsed.setText("(never been used before)");
				}

			} else {
				twLastUsed.setText("");
			}
			return v;
		}

	}

	


	private class ConnectionChangeReceiver extends BroadcastReceiver {
		@Override
		public void onReceive(Context context, Intent intent) {
			ConnectivityManager connectivityManager = (ConnectivityManager) context
					.getSystemService(Context.CONNECTIVITY_SERVICE);
			NetworkInfo activeNetInfo = connectivityManager
					.getActiveNetworkInfo();
			NetworkInfo mobNetInfo = connectivityManager
					.getNetworkInfo(ConnectivityManager.TYPE_MOBILE);
			if (activeNetInfo != null) {
				Toast.makeText(context,
						"Active Network Type : " + activeNetInfo.getTypeName(),
						Toast.LENGTH_SHORT).show();
			}
			if (mobNetInfo != null) {
				Toast.makeText(context,
						"Mobile Network Type : " + mobNetInfo.getTypeName(),
						Toast.LENGTH_SHORT).show();
			}
		}
	}
}
