package no.ntnu.item.mopenid.app;

import java.io.File;
import java.io.FileNotFoundException;
import java.net.URISyntaxException;

import no.ntnu.item.mopenid.util.BackupUtils;
import no.ntnu.item.mopenid.util.CryptoSuite;
import no.ntnu.item.mopenid.util.Utils;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.text.Editable;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

public class SelectMode extends Activity {

	private static final int DIALOG_HELP = 0;
	private static final int DIALOG_NETWORK_CONFIGURATION = 1;
	private static final int DIALOG_ENTER_DECRYPTION_KEY = 2;
	public static final int RESULT_AUTHENTICATION_CANCELLATION = 10;
	public static final int RESULT_AUTHENTICATION_SUCCESSFUL = 11;
	private BroadcastReceiver receiver;
	private Dialog currentShowingDialog;
	private int currentShowingDialogId = -1;
	private boolean receiverRegistered = false;
	private boolean firstStartup = true;
	private boolean isWaitingForStaticIPModeActivity = false,
			isWaitingFormOpenIdActivity = false;
	private String decryptionKey = null, encryptionKey = null;
	private boolean wifiNetworkReady = false;
	private boolean mobileNetworkReady = false;
	private int importMode = -1;
	private int numberOfNewIdsAdded;
	private String importUrl;
	private boolean avoidNotSupportedScreen = false;
	private InteruptHandler handler;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		setContentView(R.layout.select_mode);
		setTitle(R.string.app_full_name);
		Utils.setApplicationContext(getApplicationContext());

		setupButtons();

		Utils.saveInitialWifiState(getApplicationContext());

	}

	private void setupButtons() {

		Button staticIpButton = (Button) findViewById(R.id.staticIpButton);
		staticIpButton.setOnClickListener(new OnClickListener() {

			// @Override
			public void onClick(View v) {
				if (!Utils.isPublicAndConnectedAlready(getApplicationContext())
						&& !Utils.DEBUGMODE) {
					launchConfigurationSetupReceiverAndShowDialog();
					isWaitingForStaticIPModeActivity = true;
					setupInteruptActionIfTooLongTimeHavePassed();

				} else {
					Utils.startIJettyService();
					Intent startActivityIntent = new Intent(v.getContext(),
							StaticIpModeActivation.class);
					startActivityForResult(startActivityIntent, 0);

				}
			}
		});

		Button mopenidButton = (Button) findViewById(R.id.mopenidButton);
		mopenidButton.setOnClickListener(new OnClickListener() {

			public void onClick(View v) {
				if (!Utils.isPublicAndConnectedAlready(getApplicationContext())
						&& !Utils.DEBUGMODE) {
					launchConfigurationSetupReceiverAndShowDialog();
					setupInteruptActionIfTooLongTimeHavePassed();
					isWaitingFormOpenIdActivity = true;

				} else {
					Utils.startIJettyService();
					Intent startActivityIntent = new Intent(v.getContext(),
							SelectMOpenIDMode.class);
					startActivityForResult(startActivityIntent, 0);
				}

			}
		});

		Button helpButton = (Button) findViewById(R.id.button_help);
		helpButton.setOnClickListener(new OnClickListener() {

			public void onClick(View v) {
				showDialog(DIALOG_HELP);
			}
		});

		Button _exitButton = (Button) findViewById(R.id.button_exit);
		_exitButton.setOnClickListener(new OnClickListener() {

			public void onClick(View v) {
				finish();
			}
		});
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);

		if (resultCode == RESULT_AUTHENTICATION_SUCCESSFUL
				|| resultCode == RESULT_AUTHENTICATION_CANCELLATION) {
			avoidNotSupportedScreen = true;
			// go back to initial phone settings
			if (!Utils.DEBUGMODE) {
				Utils.restoreApnSettingToNormal(getApplicationContext());
				Utils.resetToInitialWifiState(getApplicationContext());
				wifiNetworkReady = mobileNetworkReady = false;
				Log.i("eirik", "Initial settings reset successful");

			}
			Utils.stopIJettyService();
		}
		Log.i("eirik", "in onActivityResult(). resultcode = " + resultCode);
	}

	protected void onResume() {
		super.onResume();
		setupButtons();
		handler = new InteruptHandler();

		if (firstStartup && !Utils.DEBUGMODE) {
			Log.d("eirik", "First startup TRUE");
			if (!Utils.isPublicAndConnectedAlready(getApplicationContext())
					&& !Utils.DEBUGMODE) {
				if (Utils.apnSettingsSupported()) {
					showDialog(DIALOG_NETWORK_CONFIGURATION);
					setupPublicMobileConnectedReceiver();
					setupInteruptActionIfTooLongTimeHavePassed();
					new ConfigureNetworkReachabilityTask().execute();
				} else {
					showDialog(R.layout.dialog_not_supported);
				}

			}
			firstStartup = false;
			Log.i("eirik", "performed first startup");
		} else if (!Utils.apnSettingsSupported() && !avoidNotSupportedScreen) {
			showDialog(R.layout.dialog_not_supported);
		} else if (Utils.isPublicAndConnectedAlready(getApplicationContext())
				&& !Utils.DEBUGMODE) {
			if (currentShowingDialogId == DIALOG_NETWORK_CONFIGURATION) {
				removeDialog(DIALOG_NETWORK_CONFIGURATION);
				currentShowingDialogId = -1;
				currentShowingDialog = null;
			}
		}

	}

	private class InteruptHandler extends Handler {

		@Override
		public void handleMessage(Message msg) {
			if(!(mobileNetworkReady && wifiNetworkReady)) {
				SelectMode.this.removeDialog(currentShowingDialogId);
				SelectMode.this.showDialog(R.layout.dialog_mobile_not_enabled);
			}

		}

		public void delay(long delayMillis) {

			this.removeMessages(0);

			sendMessageDelayed(obtainMessage(0), delayMillis);

		}

	};

	private void setupInteruptActionIfTooLongTimeHavePassed() {
		if (handler == null) {
			handler = new InteruptHandler();
		}
		int secondsDelay = 18;
		handler.delay(1000*secondsDelay);
	}

	public boolean onSearchRequested() {
		return false;
	}

	private void setupPublicMobileConnectedReceiver() {
		receiver = (Utils.DEBUGMODE) ? null : new BroadcastReceiver() {

			@Override
			public void onReceive(Context context, Intent intent) {

				NetworkInfo networkInfo = (NetworkInfo) intent
						.getParcelableExtra("networkInfo");
				Log.i("apn", "received connectivity change: " + networkInfo);
				if (networkInfo.getType() == ConnectivityManager.TYPE_MOBILE
						&& networkInfo.isConnected()) {
					if (networkInfo.getExtraInfo().contains(".public")) {
						mobileNetworkReady = true;
						Log.d("apn", "mobileNetworkReady = true");
					}
				}

				if (networkInfo.getType() == ConnectivityManager.TYPE_WIFI) {
					if (!networkInfo.isConnectedOrConnecting()) {
						Log.d("apn", "wifiNetworkReady = true");
						wifiNetworkReady = true;
					}
				}

				if (!Utils.isWifiEnabled()) {
					Log.d("apn", "wifiNetworkReady = true");
					wifiNetworkReady = true;
				}

				if (wifiNetworkReady
						&& mobileNetworkReady
						&& currentShowingDialog != null
						&& currentShowingDialog.isShowing()
						&& currentShowingDialogId == DIALOG_NETWORK_CONFIGURATION) {
					performButtonAction();
				}

			}
		};
		registerReceiver(receiver, new IntentFilter(
				"android.net.conn.CONNECTIVITY_CHANGE"));
		Log.i("eirik", "set up receiver");
		receiverRegistered = true;
	}

	protected void onPause() {
		super.onPause();
		Log.i("eirik", "onPause SelectMode");
	}

	protected void onStop() {
		super.onStop();
		if (!Utils.DEBUGMODE && receiverRegistered) {
			unregisterReceiver(receiver);
			receiver = null;
			receiverRegistered = false;
			Log.i("eirik",
					"in onStop() in SelectMode. performed unregistration of receiver");
		}
	}

	protected void onDestroy() {
		super.onDestroy();
		if (!Utils.DEBUGMODE) {
			Utils.restoreApnSettingToNormal(getApplicationContext());
			Utils.resetToInitialWifiState(getApplicationContext());
			firstStartup = true;
		}
		Utils.stopIJettyService();
		Log.i("eirik", "onDestroy called in selectMode");
	}

	protected Dialog onCreateDialog(int id) {
		Dialog dialog;

		switch (id) {
		case DIALOG_HELP:
			dialog = new Dialog(this);

			dialog.setContentView(R.layout.popup_help);
			dialog.setTitle("mOpenID vs Static IP mode");
			dialog.setCanceledOnTouchOutside(true);

			Button okButton = (Button) dialog.findViewById(R.id.help_ok_button);
			okButton.setOnClickListener(new OnClickListener() {

				public void onClick(View v) {
					removeDialog(DIALOG_HELP);
				}
			});
			break;

		case R.layout.dialog_mobile_not_enabled:
			dialog = new Dialog(this);
			setupDialogMobileNotEnabled(dialog);
			break;

		case DIALOG_NETWORK_CONFIGURATION:
			dialog = new Dialog(this) {
				public boolean onSearchRequested() {
					return false;
				}
			};

			dialog.setContentView(R.layout.popup_network_is_configurating);
			dialog.setTitle("Please wait");
			dialog.setCanceledOnTouchOutside(false);
			dialog.setCancelable(false);
			dialog.onSearchRequested();

			break;

		case DIALOG_ENTER_DECRYPTION_KEY:
			dialog = new Dialog(this);
			setupDecryptionKeyDialog(dialog);
			break;

		case R.layout.dialog_pasteview:
			dialog = new Dialog(this);

			setupPasteViewDialog(dialog);
			removeDialog(DIALOG_ENTER_DECRYPTION_KEY);
			break;

		case R.layout.dialog_encryption_key:
			dialog = new Dialog(this);
			setupEncryptionKeyDialog(dialog);
			break;

		case R.layout.dialog_import_confirmation:
			dialog = new Dialog(this);
			setupImportConfirmationDialog(dialog);
			removeDialog(R.layout.dialog_url);
			break;

		case R.layout.dialog_url:
			dialog = new Dialog(this);
			setupUrlDialog(dialog);
			removeDialog(DIALOG_ENTER_DECRYPTION_KEY);
			break;

		case R.layout.dialog_not_supported:
			dialog = new Dialog(this);
			setupNotSupportedDialog(dialog);
			break;

		default:
			dialog = null;
		}
		currentShowingDialog = dialog;
		currentShowingDialogId = id;
		return dialog;
	}

	private void setupDialogMobileNotEnabled(Dialog dialog) {
		dialog.setContentView(R.layout.dialog_mobile_not_enabled);
		dialog.setTitle("Mobile network failure");
		dialog.setCanceledOnTouchOutside(false);
		dialog.setCancelable(false);

		Button exitButton = (Button) dialog.findViewById(R.id.btn_exit);
		exitButton.setOnClickListener(new OnClickListener() {

			public void onClick(View v) {
				finish();
			}
		});

	}

	private void setupNotSupportedDialog(Dialog dialog) {
		dialog.setContentView(R.layout.dialog_not_supported);
		dialog.setCanceledOnTouchOutside(false);
		dialog.setCancelable(false);
		dialog.setTitle("Not supported");
		final TextView infoText = (TextView) dialog
				.findViewById(R.id.not_supported_info_text);
		String currentApnName = Utils.getCurrentApnName();
		infoText.setText(infoText.getText().toString()
				.replaceFirst("%s", currentApnName));

		Button exitButton = (Button) dialog.findViewById(R.id.btn_exit);
		exitButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				finish();
			}
		});

	}

	private void setupUrlDialog(final Dialog dialog) {
		dialog.setContentView(R.layout.dialog_url);
		dialog.setCanceledOnTouchOutside(true);
		dialog.setTitle("Import from URL");

		final EditText urlTextView = (EditText) dialog
				.findViewById(R.id.url_encrypted_ids);
		Button importButton = (Button) dialog.findViewById(R.id.button_import);
		importButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				importUrl = urlTextView.getText().toString();
				Uri uri = Uri.parse(importUrl);
				new DownloadEncryptedIdsTask().execute(uri);
			}
		});

		// set the keyboard to automatically show
		urlTextView.setOnFocusChangeListener(new View.OnFocusChangeListener() {
			public void onFocusChange(View v, boolean hasFocus) {
				if (hasFocus) {
					dialog.getWindow()
							.setSoftInputMode(
									WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_VISIBLE);
				}
			}
		});
	}

	private void setupImportConfirmationDialog(Dialog dialog) {
		dialog.setContentView(R.layout.dialog_import_confirmation);
		dialog.setCanceledOnTouchOutside(true);
		dialog.setTitle("Import successful");
		TextView informationTextView = (TextView) dialog
				.findViewById(R.id.import_information_text);

		String informationText = informationTextView.getText().toString();
		String updatedInformationText = informationText.replaceFirst("%d",
				Integer.toString(numberOfNewIdsAdded));
		informationTextView.setText(updatedInformationText);
		Log.d("eirik", "updated text: " + updatedInformationText);

		Button okayButton = (Button) dialog.findViewById(R.id.button_okay);
		okayButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				dismissDialog(R.layout.dialog_import_confirmation);
			}
		});

	}

	private void setupPasteViewDialog(Dialog dialog) {
		dialog.setContentView(R.layout.dialog_pasteview);
		dialog.setCanceledOnTouchOutside(true);
		dialog.setTitle(R.string.dialog_pasteview_title);

		final EditText encryptedTextView = (EditText) dialog
				.findViewById(R.id.pastewindow);

		Button doImportButton = (Button) dialog
				.findViewById(R.id.button_import_do);
		doImportButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View view) {
				String encryptedText = encryptedTextView.getText().toString();
				importIdsFromEncryptedString(encryptedText);
			}
		});

	}

	private void setupDecryptionKeyDialog(final Dialog dialog) {
		dialog.setContentView(R.layout.dialog_decryption_key);
		dialog.setTitle(R.string.provide_key);
		dialog.setCanceledOnTouchOutside(true);
		final EditText keyView = (EditText) dialog
				.findViewById(R.id.decryption_key);
		Button importButton = (Button) dialog.findViewById(R.id.button_import);

		// set the keyboard to automatically show
		keyView.setOnFocusChangeListener(new View.OnFocusChangeListener() {
			public void onFocusChange(View v, boolean hasFocus) {
				if (hasFocus) {
					dialog.getWindow()
							.setSoftInputMode(
									WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_VISIBLE);
				}
			}
		});

		importButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View view) {
				decryptionKey = keyView.getText().toString();
				if (importMode == R.id.from_edittextview)
					showPasteViewWithDecryptionKey(keyView.getText());
				else if (importMode == R.id.from_url)
					showUrlViewWithDecryptionKey(keyView.getText());
				else
					return;
			}
		});
	}

	private void importIdsFromEncryptedString(String encryptedText) {
		String decryptedText = null;
		try {
			Log.d("eirik", "encrypted text: " + encryptedText);
			decryptedText = CryptoSuite.decrypt(decryptionKey, encryptedText);
		} catch (Exception e) {
			Log.d("eirik",
					"exception happened when tried to decrypt (probably wrong key): "
							+ e.getMessage());
			dismissDialog(currentShowingDialogId);

			final AlertDialog alertDialog = new AlertDialog.Builder(
					SelectMode.this).create();
			alertDialog.setIcon(R.drawable.ic_delete);
			alertDialog.setTitle("Message");
			alertDialog
					.setMessage("An error occured when trying to decrypt. Is you decryption key correct?");
			alertDialog.setButton("Dismiss dialog",
					new DialogInterface.OnClickListener() {
						public void onClick(DialogInterface dialog, int which) {
							alertDialog.dismiss();
						}
					});
			alertDialog.show();

			e.printStackTrace();
		}
		if (Utils.isCorrectDecryptionKey(decryptedText)) {
			Log.d("eirik", "correct decryption key");
			numberOfNewIdsAdded = BackupUtils
					.addNewImportedIdsFromDecryptedtext(decryptedText);
			dismissDialog(currentShowingDialogId);
			showDialog(R.layout.dialog_import_confirmation);
		} else {
			Log.d("eirik", "wrong decryptionkey");
		}

		Log.d("eirik", "decrypted text: " + decryptedText);
	}

	private void setupEncryptionKeyDialog(final Dialog dialog) {
		dialog.setContentView(R.layout.dialog_encryption_key);
		dialog.setTitle(R.string.provide_enc_key);
		dialog.setCanceledOnTouchOutside(true);
		final EditText keyView = (EditText) dialog
				.findViewById(R.id.encryption_key);
		Button importButton = (Button) dialog.findViewById(R.id.button_export);

		// set the keyboard to automatically show
		keyView.setOnFocusChangeListener(new View.OnFocusChangeListener() {
			public void onFocusChange(View v, boolean hasFocus) {
				if (hasFocus) {
					dialog.getWindow()
							.setSoftInputMode(
									WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_VISIBLE);
				}
			}
		});

		importButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View view) {
				dismissDialog(R.layout.dialog_encryption_key);
				encryptionKey = keyView.getText().toString();
				encryptAndExportIds(encryptionKey);
			}
		});
	}

	protected void showUrlViewWithDecryptionKey(Editable text) {
		removeDialog(R.id.layout_decryption_key);
		showDialog(R.layout.dialog_url);
	}

	protected void showPasteViewWithDecryptionKey(Editable text) {
		removeDialog(R.id.layout_decryption_key);
		showDialog(R.layout.dialog_pasteview);
	}

	private void launchConfigurationSetupReceiverAndShowDialog() {
		showDialog(DIALOG_NETWORK_CONFIGURATION);
		if (!receiverRegistered) {
			setupPublicMobileConnectedReceiver();
			Log.i("eirik", "set up receiver in the second place");
			receiverRegistered = true;
		}
		new ConfigureNetworkReachabilityTask().execute();

	}

	private void performButtonAction() {
		Log.d("eirik", "in performButtonAction() method");
		removeDialog(DIALOG_NETWORK_CONFIGURATION);
		currentShowingDialog = null;
		currentShowingDialogId = -1;

		if (isWaitingFormOpenIdActivity || isWaitingForStaticIPModeActivity) {
			Utils.startIJettyService();
		}
		if (isWaitingFormOpenIdActivity) {
			isWaitingFormOpenIdActivity = false;
			Intent startActivityIntent = new Intent(getApplicationContext(),
					SelectMOpenIDMode.class);
			startActivityForResult(startActivityIntent, 0);
		} else if (isWaitingForStaticIPModeActivity) {
			isWaitingForStaticIPModeActivity = false;
			Intent startActivityIntent = new Intent(getApplicationContext(),
					StaticIpModeActivation.class);
			startActivityForResult(startActivityIntent, 0);
		}
	}

	private class ConfigureNetworkReachabilityTask extends
			AsyncTask<Void, Void, Void> {

		@Override
		protected Void doInBackground(Void... params) {

			changeAPNSettingsToObtainPublicIPAddress();
			Utils.disableWifi(getApplicationContext());
			return null;
		}

		private void changeAPNSettingsToObtainPublicIPAddress() {
			Utils.saveCurrentAndSwitchToPublicAPN(getApplicationContext());
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.select_mode_menu, menu);
		return super.onCreateOptionsMenu(menu);

	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		// Handle item selection
		switch (item.getItemId()) {
		case R.id.export_option:
			showDialog(R.layout.dialog_encryption_key);
			return true;

		case R.id.import_option:
			// showing submenu - do nothing
			return true;

		case R.id.from_edittextview:
			Log.d("eirik", "show paste window");
			importMode = R.id.from_edittextview;
			showDialog(DIALOG_ENTER_DECRYPTION_KEY);
			return true;

		case R.id.from_url:
			importMode = R.id.from_url;
			showDialog(DIALOG_ENTER_DECRYPTION_KEY);
			Log.d("eirik", "show url window");
			return true;

		default:
			return super.onOptionsItemSelected(item);
		}
	}

	private void encryptAndExportIds(String key) {
		String savedFilename = BackupUtils
				.encryptOpenIdsWithPasswordAndWriteToSDCard(key);

		if (savedFilename != null) {
			Intent intent = new Intent(Intent.ACTION_SEND);
			intent.putExtra(Intent.EXTRA_STREAM, Uri.fromFile(new File(
					"/mnt/sdcard/mopenid/", savedFilename)));
			intent.setType("text/plain");
			// intent.get
			Intent actionChooserIntent = Intent.createChooser(intent,
					"Export to:");
			startActivity(actionChooserIntent);
		} else {

		}

	}

	private class DownloadEncryptedIdsTask extends AsyncTask<Uri, Void, String> {

		protected String doInBackground(Uri... uris) {

			try {
				return Utils.fetchStringFromUri(uris[0]);
			} catch (URISyntaxException e) {
				Log.d("eirik", "URISyntaxException when trying to parse file");
				e.printStackTrace();
				return null;
			} catch (FileNotFoundException e) {
				Log.d("eirik",
						"FileNotFoundException when trying to parse file");
				e.printStackTrace();
				return null;
			}
		}

		protected void onPostExecute(String result) {

			Log.d("eirik", "got the encrypted string: " + result);
			importIdsFromEncryptedString(result);

		}

	}
}