package no.ntnu.item.mopenid.app;

import no.ntnu.item.mopenid.util.Utils;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;

public class StaticIpModeActivation extends AbstractActivation {

	private static final String INTENT_ACTION_SET_STATIC_IP_MODE = "no.ntnu.item.mopenid.ACTION_STATIC_IP_MODE";
	private static final String INTENT_ACTION_SET_STATIC_IP_MODE_REQUEST = "no.ntnu.item.mopenid.ACTION_STATIC_IP_MODE_REQUEST";
	private BroadcastReceiver receiver;

	
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		identifier = Utils.getLocalEndpointURLAsString();
		
	}
	
	@Override
	public void onStart() {
		super.onStart();
		registerModeResolverListener();
	}

	private void registerModeResolverListener() {
		receiver = new BroadcastReceiver() {
			
			@Override
			public void onReceive(Context context, Intent intent) {
				sendIntentToServlet(true);
			}
		};
		
		registerReceiver(receiver, new IntentFilter(INTENT_ACTION_SET_STATIC_IP_MODE_REQUEST));
		
	}

	private void sendIntentToServlet(boolean isStatic) {
		Intent intent = new Intent(INTENT_ACTION_SET_STATIC_IP_MODE);
		intent.putExtra("staticMode", isStatic);
		sendBroadcast(intent);
	}
	
	@Override
	protected void onResume() {
		super.onResume();
		sendIntentToServlet(true);
		setAllTextViewsToSuccessful();
		showOpenIDURLAndCancelButton();
	}
	
	@Override
	protected void onPause() {
		super.onPause();
		unregisterReceiver(receiver);
		sendIntentToServlet(false);
		
	}
	
	@Override
	protected void setUpCancelButtonBehavior() {
		Button cancelButton = (Button) findViewById(R.id.cancelButton);
		cancelButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				Log.i("eirik", "Cancel button clicked");
				perfromCancelAction();
			}
		});
		
	}

	@Override
	protected void setupProgressDialog() {
		// Do nothing
		
	}

	@Override
	protected void perfromCancelAction() {
		removeDialog(currentShowingDialogId);
		setResult(SelectMode.RESULT_AUTHENTICATION_CANCELLATION);
		finish();
		
	}

	@Override
	protected void authenticationFinished() {
		removeDialog(AUTHENTICATION_CONFIRMATION_DIALOG);
		setResult(SelectMode.RESULT_AUTHENTICATION_SUCCESSFUL);
		finish();
	}

	@Override
	protected void showOpenIDURLAndCancelButton() {
		TextView usageInformationTextView = (TextView) findViewById(R.id.usage_instructions), openidURLTextView = (TextView) findViewById(R.id.openid_to_use);
		usageInformationTextView.setVisibility(View.VISIBLE);
		openidURLTextView.setText(Utils.getLocalEndpointURLAsString());
		openidURLTextView.setVisibility(View.VISIBLE);
		Button cancelButton = (Button) findViewById(R.id.cancelButton);
		cancelButton.setVisibility(View.VISIBLE);
	}
	
	
}
