package no.ntnu.item.mopenid.util;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import android.content.ContentValues;
import android.os.Environment;
import android.text.format.DateFormat;
import android.util.Log;

public class BackupUtils {
	
	
	public static final String ENC_FILE_NAME = "openids_enc_";
	public static final String OKAY_FLAG = "iamdecrypted";

	public static String encryptOpenIdsWithPasswordAndWriteToSDCard(String encryptionKey) {
		boolean writeOk = false, encryptOk = false;
		String savedFilename = null;
		List<OpenID> savedOpenIDs = Utils.getSavedOpenIDs();
		
		if(savedOpenIDs.size() == 0)
			return null;
		
		String unEncryptedString = createStringFromList(savedOpenIDs);
		Log.d("eirik", "formatted openids ready for encryption: "+unEncryptedString);
		
		try {
			String encryptedString = CryptoSuite.encrypt(encryptionKey, unEncryptedString);
			encryptOk = true;
			String time = timestamp();
			savedFilename = ENC_FILE_NAME + time +".txt";
			writeOk = writeFileToSD(savedFilename, encryptedString);
			Log.d("eirik", "Written to file: "+writeOk+ ". Encrypted: "+encryptOk);
			
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			Log.d("eirik", e.toString());
		}
		
		return savedFilename;
	}

	private static String timestamp() {
		return DateFormat.format("dd.MM.yyyy-kkmmss", new Date()).toString();
		
	}

	private static String createStringFromList(List<OpenID> savedOpenIDs) {
		StringBuilder result = new StringBuilder();
		result.append(OKAY_FLAG+"\n");
		for (OpenID openId : savedOpenIDs) {
			result.append(openId.toFormattedString());
			result.append("\n");
		}
		return result.toString();
	}

	private static boolean writeFileToSD(String sFileName, String sBody) {
		try {
			File root = new File(Environment.getExternalStorageDirectory(),
					"mopenid");
			if (!root.exists()) {
				root.mkdirs();
			}
			File file = new File(root, sFileName);
			FileWriter writer = new FileWriter(file);
			writer.append(sBody);
			writer.flush();
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}

	public static int addNewImportedIdsFromDecryptedtext(String decryptedText) {
		String[] lines = decryptedText.split("\n");
		ArrayList<OpenID> ids = new ArrayList<OpenID>();
		for (int i = 1; i < lines.length; i++) {
			String[] row = lines[i].split(";");
			OpenID openIdAtCurrentRow = new OpenID(row[0]);
			openIdAtCurrentRow.setLastUsed(row[1]);
			openIdAtCurrentRow.setSecurityToken(row[2]);
			openIdAtCurrentRow.setCreated(row[3]);
			openIdAtCurrentRow.setDescription(row[4]);
			ids.add(openIdAtCurrentRow);
		}
		int numberOfInsertedRows = 0;
		for (Iterator iterator = ids.iterator(); iterator.hasNext();) {
			OpenID openID = (OpenID) iterator.next();

			if(!Utils.idAlreadyExists(openID)) {
				if(Utils.insertNewIdToDatabase(openID)) {
					numberOfInsertedRows++;
				}
			} 
				
		}
		
		return numberOfInsertedRows;
	}

	
}
